@@ -1,9 +1,8 @@
 from . import base
-from . import momento_cachedb
+
 from . import redis_cachedb
 
 __all__ = [
     "base",
-    "momento_cachedb",
     "redis_cachedb",
 ]
