@@ -7,25 +7,26 @@ import tempfile
 import time
 from collections import deque
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 from urllib.parse import urlparse
 
-from bs4 import BeautifulSoup
 from dotenv import load_dotenv
-from github import Github
-from github.ContentFile import ContentFile
-from github.Label import Label
-from github.Repository import Repository
-from pydantic import BaseModel, BaseSettings, Field
+
+if TYPE_CHECKING:
+    from github import Github
+    from github.ContentFile import ContentFile
+    from github.Label import Label
+    from github.Repository import Repository
 
 from langroid.mytypes import DocMetaData, Document
-from langroid.parsing.document_parser import DocumentParser
+from langroid.parsing.document_parser import DocumentParser, DocumentType
 from langroid.parsing.parser import Parser, ParsingConfig
+from langroid.pydantic_v1 import BaseModel, BaseSettings, Field
 
 logger = logging.getLogger(__name__)
 
 
-def _get_decoded_content(content_file: ContentFile) -> str:
+def _get_decoded_content(content_file: "ContentFile") -> str:
     if content_file.encoding == "base64":
         return content_file.decoded_content.decode("utf-8") or ""
     elif content_file.encoding == "none":
@@ -55,7 +56,7 @@ class IssueData(BaseModel):
     text: str = Field(..., description="Text of issue, i.e. description body")
 
 
-def get_issue_size(labels: List[Label]) -> str | None:
+def get_issue_size(labels: List["Label"]) -> str | None:
     sizes = ["XS", "S", "M", "L", "XL", "XXL"]
     return next((label.name for label in labels if label.name in sizes), None)
 
@@ -118,6 +119,8 @@ class RepoLoader:
         self.config = config
         self.clone_path: Optional[str] = None
         self.log_file = ".logs/repo_loader/download_log.json"
+        self.repo: Optional["Repository"] = None  # Initialize repo as Optional
+
         os.makedirs(os.path.dirname(self.log_file), exist_ok=True)
         if not os.path.exists(self.log_file):
             with open(self.log_file, "w") as f:
@@ -128,20 +131,25 @@ class RepoLoader:
             logger.info(f"Repo Already downloaded in {log[self.url]}")
             self.clone_path = log[self.url]
 
+        # it's a core dependency, so we don't need to enclose in try/except
+        from github import Github  # Late import
+
+        load_dotenv()
+        # authenticated calls to github api have higher rate limit
+        token = os.getenv("GITHUB_ACCESS_TOKEN")
+
         if "github.com" in self.url:
             repo_name = self.url.split("github.com/")[1]
         else:
             repo_name = self.url
-        load_dotenv()
-        # authenticated calls to github api have higher rate limit
-        token = os.getenv("GITHUB_ACCESS_TOKEN")
+
         g = Github(token)
         self.repo = self._get_repo_with_retry(g, repo_name)
 
     @staticmethod
     def _get_repo_with_retry(
-        g: Github, repo_name: str, max_retries: int = 5
-    ) -> Repository:
+        g: "Github", repo_name: str, max_retries: int = 5
+    ) -> "Repository":
         """
         Get a repo from the GitHub API, retrying if the request fails,
         with exponential backoff.
@@ -174,6 +182,10 @@ class RepoLoader:
 
     def get_issues(self, k: int | None = 100) -> List[IssueData]:
         """Get up to k issues from the GitHub repo."""
+        if self.repo is None:
+            logger.warning("No repo found. Ensure the URL is correct.")
+            return []  # Return an empty list rather than raise an error in this case
+
         if k is None:
             issues = self.repo.get_issues(state="all")
         else:
@@ -225,7 +237,7 @@ class RepoLoader:
         """
         return file_type not in self.config.non_code_types
 
-    def _is_allowed(self, content: ContentFile) -> bool:
+    def _is_allowed(self, content: "ContentFile") -> bool:
         """
         Check if a file or directory content is allowed to be included.
 
@@ -302,6 +314,10 @@ class RepoLoader:
             Dict[str, Union[str, List[Dict]]]:
             A dictionary containing file and directory names, with file contents.
         """
+        if self.repo is None:
+            logger.warning("No repo found. Ensure the URL is correct.")
+            return {}  # Return an empty dict rather than raise an error in this case
+
         root_contents = self.repo.get_contents("")
         if not isinstance(root_contents, list):
             root_contents = [root_contents]
@@ -491,18 +507,25 @@ class RepoLoader:
 
     @staticmethod
     def get_documents(
-        path: str,
+        path: str | bytes,
         parser: Parser = Parser(ParsingConfig()),
         file_types: Optional[List[str]] = None,
         exclude_dirs: Optional[List[str]] = None,
         depth: int = -1,
         lines: Optional[int] = None,
+        doc_type: str | DocumentType | None = None,
     ) -> List[Document]:
         """
         Recursively get all files under a path as Document objects.
 
         Args:
-            path (str): The path to the directory or file.
+            path (str|bytes): The path to the directory or file, or bytes content.
+                The bytes option is meant to support the case where the content
+                has already been read from a file in an upstream process
+                (e.g. from an API or a database), and we want to avoid having to
+                write it to a temporary file just to read it again.
+                (which can be very slow for large files,
+                especially in a docker container)
             parser (Parser): Parser to use to parse files.
             file_types (List[str], optional): List of file extensions OR
                 filenames OR file_path_names to  include.
@@ -513,60 +536,48 @@ class RepoLoader:
                 which includes all depths.
             lines (int, optional): Number of lines to read from each file.
                 Defaults to None, which reads all lines.
-
+            doc_type (str|DocumentType | None, optional): The type of document to parse.
         Returns:
             List[Document]: List of Document objects representing files.
 
         """
         docs = []
         file_paths = []
-        path_obj = Path(path).resolve()
-
-        if path_obj.is_file():
-            file_paths.append(str(path_obj))
+        if isinstance(path, bytes):
+            file_paths.append(path)
         else:
-            path_depth = len(path_obj.parts)
-            for root, dirs, files in os.walk(path):
-                # Exclude directories if needed
-                if exclude_dirs:
-                    dirs[:] = [d for d in dirs if d not in exclude_dirs]
+            path_obj = Path(path).resolve()
 
-                current_depth = len(Path(root).resolve().parts) - path_depth
-                if depth == -1 or current_depth <= depth:
-                    for file in files:
-                        file_path = str(Path(root) / file)
-                        if (
-                            file_types is None
-                            or RepoLoader._file_type(file_path) in file_types
-                            or os.path.basename(file_path) in file_types
-                            or file_path in file_types
-                        ):
-                            file_paths.append(file_path)
+            if path_obj.is_file():
+                file_paths.append(str(path_obj))
+            else:
+                path_depth = len(path_obj.parts)
+                for root, dirs, files in os.walk(path):
+                    # Exclude directories if needed
+                    if exclude_dirs:
+                        dirs[:] = [d for d in dirs if d not in exclude_dirs]
+
+                    current_depth = len(Path(root).resolve().parts) - path_depth
+                    if depth == -1 or current_depth <= depth:
+                        for file in files:
+                            file_path = str(Path(root) / file)
+                            if (
+                                file_types is None
+                                or RepoLoader._file_type(file_path) in file_types
+                                or os.path.basename(file_path) in file_types
+                                or file_path in file_types
+                            ):
+                                file_paths.append(file_path)
 
         for file_path in file_paths:
-            _, file_extension = os.path.splitext(file_path)
-            if file_extension.lower() in [".pdf", ".docx", ".doc"]:
-                doc_parser = DocumentParser.create(
+            docs.extend(
+                DocumentParser.chunks_from_path_or_bytes(
                     file_path,
-                    parser.config,
+                    parser,
+                    doc_type=doc_type,
+                    lines=lines,
                 )
-                docs.extend(doc_parser.get_doc_chunks())
-            else:
-                with open(file_path, "r") as f:
-                    if lines is not None:
-                        file_lines = list(itertools.islice(f, lines))
-                        content = "\n".join(line.strip() for line in file_lines)
-                    else:
-                        content = f.read()
-                soup = BeautifulSoup(content, "html.parser")
-                text = soup.get_text()
-                docs.append(
-                    Document(
-                        content=text,
-                        metadata=DocMetaData(source=str(file_path)),
-                    )
-                )
-
+            )
         return docs
 
     def load_docs_from_github(
@@ -589,6 +600,10 @@ class RepoLoader:
             list of Document objects, each has fields `content` and `metadata`,
             and `metadata` has fields `url`, `filename`, `extension`, `language`
         """
+        if self.repo is None:
+            logger.warning("No repo found. Ensure the URL is correct.")
+            return []  # Return an empty list rather than raise an error
+
         contents = self.repo.get_contents("")
         if not isinstance(contents, list):
             contents = [contents]
