@@ -9,10 +9,10 @@ from urllib.parse import urldefrag, urljoin, urlparse
 import fire
 import requests
 from bs4 import BeautifulSoup
-from pydantic import BaseModel, HttpUrl, ValidationError, parse_obj_as
 from rich import print
 from rich.prompt import Prompt
-from trafilatura.spider import focused_crawler
+
+from langroid.pydantic_v1 import BaseModel, HttpUrl, ValidationError, parse_obj_as
 
 logger = logging.getLogger(__name__)
 
@@ -112,26 +112,35 @@ def is_url(s: str) -> bool:
         return False
 
 
-def get_urls_and_paths(inputs: List[str]) -> Tuple[List[str], List[str]]:
+def get_urls_paths_bytes_indices(
+    inputs: List[str | bytes],
+) -> Tuple[List[int], List[int], List[int]]:
     """
-    Given a list of inputs, return a list of URLs and a list of paths.
+    Given a list of inputs, return a
+    list of indices of URLs, list of indices of paths, list of indices of byte-contents.
     Args:
-        inputs: list of strings
+        inputs: list of strings or bytes
     Returns:
-        list of URLs, list of paths
+        list of Indices of URLs,
+        list of indices of paths,
+        list of indices of byte-contents
     """
     urls = []
     paths = []
-    for item in inputs:
+    byte_list = []
+    for i, item in enumerate(inputs):
+        if isinstance(item, bytes):
+            byte_list.append(i)
+            continue
         try:
-            m = Url(url=parse_obj_as(HttpUrl, item))
-            urls.append(str(m.url))
+            Url(url=parse_obj_as(HttpUrl, item))
+            urls.append(i)
         except ValidationError:
             if os.path.exists(item):
-                paths.append(item)
+                paths.append(i)
             else:
                 logger.warning(f"{item} is neither a URL nor a path.")
-    return urls, paths
+    return urls, paths, byte_list
 
 
 def crawl_url(url: str, max_urls: int = 1) -> List[str]:
@@ -140,6 +149,8 @@ def crawl_url(url: str, max_urls: int = 1) -> List[str]:
     up to a maximum of `max_urls`.
     This has not been tested to work as intended. Ignore.
     """
+    from trafilatura.spider import focused_crawler
+
     if max_urls == 1:
         # no need to crawl, just return the original list
         return [url]
@@ -211,10 +222,15 @@ def find_urls(
         response = requests.get(url, timeout=5)
         response.raise_for_status()
         soup = BeautifulSoup(response.text, "html.parser")
-        links = [urljoin(url, a["href"]) for a in soup.find_all("a", href=True)]
+        links = [
+            urljoin(url, a["href"])  # type: ignore
+            for a in soup.find_all("a", href=True)
+        ]
 
         # Defrag links: discard links that are to portions of same page
-        defragged_links = list(set(urldefrag(link).url for link in links))
+        defragged_links = list(
+            set(urldefrag(link).url for link in links)  # type: ignore
+        )
 
         # Filter links based on domain matching requirement
         domain_matching_links = [
