@@ -1,19 +1,21 @@
 import copy
 import logging
 from abc import ABC, abstractmethod
-from typing import Dict, List, Optional, Sequence, Tuple
+from typing import Dict, List, Optional, Sequence, Tuple, Type
 
 import numpy as np
 import pandas as pd
-from pydantic import BaseSettings
 
 from langroid.embedding_models.base import EmbeddingModel, EmbeddingModelsConfig
 from langroid.embedding_models.models import OpenAIEmbeddingsConfig
-from langroid.mytypes import Document
+from langroid.mytypes import DocMetaData, Document, EmbeddingFunction
+from langroid.pydantic_v1 import BaseSettings
 from langroid.utils.algorithms.graph import components, topological_sort
 from langroid.utils.configuration import settings
+from langroid.utils.object_registry import ObjectRegistry
 from langroid.utils.output.printing import print_long_text
-from langroid.utils.pandas_utils import stringify
+from langroid.utils.pandas_utils import sanitize_command, stringify
+from langroid.utils.pydantic_utils import flatten_dict
 
 logger = logging.getLogger(__name__)
 
@@ -28,10 +30,15 @@ class VectorStoreConfig(BaseSettings):
     embedding: EmbeddingModelsConfig = OpenAIEmbeddingsConfig(
         model_type="openai",
     )
+    embedding_model: Optional[EmbeddingModel] = None
     timeout: int = 60
     host: str = "127.0.0.1"
     port: int = 6333
+    # used when parsing search results back as Document objects
+    document_class: Type[Document] = Document
+    metadata_class: Type[DocMetaData] = DocMetaData
     # compose_file: str = "langroid/vector_store/docker-compose-qdrant.yml"
+    full_eval: bool = False  # runs eval without sanitization. Use only on trusted input
 
 
 class VectorStore(ABC):
@@ -41,26 +48,36 @@ class VectorStore(ABC):
 
     def __init__(self, config: VectorStoreConfig):
         self.config = config
-        self.embedding_model = EmbeddingModel.create(config.embedding)
+        if config.embedding_model is None:
+            self.embedding_model = EmbeddingModel.create(config.embedding)
+        else:
+            self.embedding_model = config.embedding_model
+        self.embedding_fn: EmbeddingFunction = self.embedding_model.embedding_fn()
 
     @staticmethod
     def create(config: VectorStoreConfig) -> Optional["VectorStore"]:
         from langroid.vector_store.chromadb import ChromaDB, ChromaDBConfig
         from langroid.vector_store.lancedb import LanceDB, LanceDBConfig
         from langroid.vector_store.meilisearch import MeiliSearch, MeiliSearchConfig
-        from langroid.vector_store.momento import MomentoVI, MomentoVIConfig
+        from langroid.vector_store.pineconedb import PineconeDB, PineconeDBConfig
+        from langroid.vector_store.postgres import PostgresDB, PostgresDBConfig
         from langroid.vector_store.qdrantdb import QdrantDB, QdrantDBConfig
+        from langroid.vector_store.weaviatedb import WeaviateDB, WeaviateDBConfig
 
         if isinstance(config, QdrantDBConfig):
             return QdrantDB(config)
         elif isinstance(config, ChromaDBConfig):
             return ChromaDB(config)
-        elif isinstance(config, MomentoVIConfig):
-            return MomentoVI(config)
         elif isinstance(config, LanceDBConfig):
             return LanceDB(config)
         elif isinstance(config, MeiliSearchConfig):
             return MeiliSearch(config)
+        elif isinstance(config, PostgresDBConfig):
+            return PostgresDB(config)
+        elif isinstance(config, WeaviateDBConfig):
+            return WeaviateDB(config)
+        elif isinstance(config, PineconeDBConfig):
+            return PineconeDB(config)
 
         else:
             logger.warning(
@@ -74,6 +91,10 @@ class VectorStore(ABC):
             )
             return None
 
+    @property
+    def embedding_dim(self) -> int:
+        return len(self.embedding_fn(["test"])[0])
+
     @abstractmethod
     def clear_empty_collections(self) -> int:
         """Clear all empty collections in the vector store.
@@ -112,8 +133,9 @@ class VectorStore(ABC):
         """
 
         self.config.collection_name = collection_name
-        if collection_name not in self.list_collections() or replace:
-            self.create_collection(collection_name, replace=replace)
+        self.config.replace_collection = replace
+        if replace:
+            self.create_collection(collection_name, replace=True)
 
     @abstractmethod
     def create_collection(self, collection_name: str, replace: bool = False) -> None:
@@ -132,21 +154,23 @@ class VectorStore(ABC):
     def compute_from_docs(self, docs: List[Document], calc: str) -> str:
         """Compute a result on a set of documents,
         using a dataframe calc string like `df.groupby('state')['income'].mean()`.
+
+        If full_eval is False (default), the input expression is sanitized to prevent
+        most common code injection attack vectors.
+        If full_eval is True, sanitization is bypassed - use only with trusted input!
         """
-        # docs may be missing some fields since they may have come from
-        # DocChatAgent retrieval, so we use the ids to get the full docs
-        ids = [str(doc.id()) for doc in docs]
-        docs = self.get_documents_by_ids(ids)
-        dicts = [doc.dict() for doc in docs]
+        # convert each doc to a dict, using dotted paths for nested fields
+        dicts = [flatten_dict(doc.dict(by_alias=True)) for doc in docs]
         df = pd.DataFrame(dicts)
 
         try:
-            result = pd.eval(  # safer than eval but limited to single expression
-                calc,
-                engine="python",
-                parser="pandas",
-                local_dict={"df": df},
-            )
+            # SECURITY MITIGATION: Eval input is sanitized to prevent most common
+            # code injection attack vectors when full_eval is False.
+            vars = {"df": df}
+            if not self.config.full_eval:
+                calc = sanitize_command(calc)
+            code = compile(calc, "<calc>", "eval")
+            result = eval(code, vars, {})
         except Exception as e:
             # return error message so LLM can fix the calc string if needed
             err = f"""
@@ -167,7 +191,7 @@ class VectorStore(ABC):
         vecdbs don't like having blank ids."""
         for d in documents:
             if d.metadata.id in [None, ""]:
-                d.metadata.id = d._unique_hash_id()
+                d.metadata.id = ObjectRegistry.new_id()
 
     @abstractmethod
     def similar_texts_with_scores(
@@ -254,11 +278,11 @@ class VectorStore(ABC):
             metadata = copy.deepcopy(id2metadata[w[0]])
             metadata.window_ids = w
             document = Document(
-                content=" ".join([d.content for d in self.get_documents_by_ids(w)]),
+                content="".join([d.content for d in self.get_documents_by_ids(w)]),
                 metadata=metadata,
             )
             # make a fresh id since content is in general different
-            document.metadata.id = document.hash_id(document.content)
+            document.metadata.id = ObjectRegistry.new_id()
             final_docs += [document]
             final_scores += [max(id2max_score[id] for id in w)]
         return list(zip(final_docs, final_scores))
