@@ -1,31 +1,47 @@
+from __future__ import annotations
+
 import logging
-from typing import Any, Dict, Generator, List, Optional, Sequence, Tuple, Type
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+)
 
-import lancedb
 import pandas as pd
 from dotenv import load_dotenv
-from lancedb.pydantic import LanceModel, Vector
-from lancedb.query import LanceVectorQueryBuilder
-from pydantic import BaseModel, ValidationError, create_model
+
+from langroid.pydantic_v1 import BaseModel, ValidationError, create_model
+
+if TYPE_CHECKING:
+    from lancedb.query import LanceVectorQueryBuilder
 
 from langroid.embedding_models.base import (
-    EmbeddingModel,
     EmbeddingModelsConfig,
 )
 from langroid.embedding_models.models import OpenAIEmbeddingsConfig
-from langroid.mytypes import Document, EmbeddingFunction
+from langroid.exceptions import LangroidImportError
+from langroid.mytypes import Document
 from langroid.utils.configuration import settings
 from langroid.utils.pydantic_utils import (
     dataframe_to_document_model,
     dataframe_to_documents,
-    extend_document_class,
-    extra_metadata,
-    flatten_pydantic_instance,
-    flatten_pydantic_model,
-    nested_dict_from_flat,
 )
 from langroid.vector_store.base import VectorStore, VectorStoreConfig
 
+try:
+    import lancedb
+    from lancedb.pydantic import LanceModel, Vector
+
+    has_lancedb = True
+except ImportError:
+    has_lancedb = False
+
 logger = logging.getLogger(__name__)
 
 
@@ -35,24 +51,19 @@ class LanceDBConfig(VectorStoreConfig):
     storage_path: str = ".lancedb/data"
     embedding: EmbeddingModelsConfig = OpenAIEmbeddingsConfig()
     distance: str = "cosine"
-    # document_class is used to store in lancedb with right schema,
-    # and also to retrieve the right type of Documents when searching.
-    document_class: Type[Document] = Document
-    flatten: bool = False  # flatten Document class into LanceSchema ?
 
 
 class LanceDB(VectorStore):
     def __init__(self, config: LanceDBConfig = LanceDBConfig()):
         super().__init__(config)
+        if not has_lancedb:
+            raise LangroidImportError("lancedb", "lancedb")
+
         self.config: LanceDBConfig = config
-        emb_model = EmbeddingModel.create(config.embedding)
-        self.embedding_fn: EmbeddingFunction = emb_model.embedding_fn()
-        self.embedding_dim = emb_model.embedding_dims
         self.host = config.host
         self.port = config.port
         self.is_from_dataframe = False  # were docs ingested from a dataframe?
         self.df_metadata_columns: List[str] = []  # metadata columns from dataframe
-        self._setup_schemas(config.document_class)
 
         load_dotenv()
         if self.config.cloud:
@@ -78,23 +89,6 @@ class LanceDB(VectorStore):
                     uri=new_storage_path,
                 )
 
-        # Note: Only create collection if a non-null collection name is provided.
-        # This is useful to delay creation of vecdb until we have a suitable
-        # collection name (e.g. we could get it from the url or folder path).
-        if config.collection_name is not None:
-            self.create_collection(
-                config.collection_name, replace=config.replace_collection
-            )
-
-    def _setup_schemas(self, doc_cls: Type[Document] | None) -> None:
-        doc_cls = doc_cls or self.config.document_class
-        self.unflattened_schema = self._create_lance_schema(doc_cls)
-        self.schema = (
-            self._create_flat_lance_schema(doc_cls)
-            if self.config.flatten
-            else self.unflattened_schema
-        )
-
     def clear_empty_collections(self) -> int:
         coll_names = self.list_collections()
         n_deletes = 0
@@ -125,7 +119,7 @@ class LanceDB(VectorStore):
             self.client.drop_table(name)
         logger.warning(
             f"""
-            Deleted {n_empty_deletes} empty collections and 
+            Deleted {n_empty_deletes} empty collections and
             {n_non_empty_deletes} non-empty collections.
             """
         )
@@ -139,7 +133,7 @@ class LanceDB(VectorStore):
         Args:
             empty (bool, optional): Whether to include empty collections.
         """
-        colls = self.client.table_names()
+        colls = self.client.table_names(limit=None)
         if len(colls) == 0:
             return []
         if empty:  # include empty tbls
@@ -149,6 +143,8 @@ class LanceDB(VectorStore):
 
     def _create_lance_schema(self, doc_cls: Type[Document]) -> Type[BaseModel]:
         """
+        NOTE: NOT USED, but leaving it here as it may be useful.
+
         Create a subclass of LanceModel with fields:
          - id (str)
          - Vector field that has dims equal to
@@ -170,13 +166,19 @@ class LanceDB(VectorStore):
         if not issubclass(doc_cls, Document):
             raise ValueError("DocClass must be a subclass of Document")
 
+        if not has_lancedb:
+            raise LangroidImportError("lancedb", "lancedb")
+
         n = self.embedding_dim
 
         # Prepare fields for the new model
         fields = {"id": (str, ...), "vector": (Vector(n), ...)}
 
+        sorted_fields = dict(
+            sorted(doc_cls.__fields__.items(), key=lambda item: item[0])
+        )
         # Add both statically and dynamically defined fields from doc_cls
-        for field_name, field in doc_cls.__fields__.items():
+        for field_name, field in sorted_fields.items():
             fields[field_name] = (field.outer_type_, field.default)
 
         # Create the new model with dynamic fields
@@ -185,40 +187,11 @@ class LanceDB(VectorStore):
         )  # type: ignore
         return NewModel  # type: ignore
 
-    def _create_flat_lance_schema(self, doc_cls: Type[Document]) -> Type[BaseModel]:
-        """
-        Flat version of the lance_schema, as nested Pydantic schemas are not yet
-        supported by LanceDB.
-        """
-        lance_model = self._create_lance_schema(doc_cls)
-        FlatModel = flatten_pydantic_model(lance_model, base_model=LanceModel)
-        return FlatModel
-
     def create_collection(self, collection_name: str, replace: bool = False) -> None:
-        """
-        Create a collection with the given name, optionally replacing an existing
-            collection if `replace` is True.
-        Args:
-            collection_name (str): Name of the collection to create.
-            replace (bool): Whether to replace an existing collection
-                with the same name. Defaults to False.
-        """
+        self.config.replace_collection = replace
         self.config.collection_name = collection_name
-        collections = self.list_collections()
-        if collection_name in collections:
-            coll = self.client.open_table(collection_name)
-            if coll.head().shape[0] > 0:
-                logger.warning(f"Non-empty Collection {collection_name} already exists")
-                if not replace:
-                    logger.warning("Not replacing collection")
-                    return
-                else:
-                    logger.warning("Recreating fresh collection")
-        self.client.create_table(collection_name, schema=self.schema, mode="overwrite")
-        if settings.debug:
-            level = logger.getEffectiveLevel()
-            logger.setLevel(logging.INFO)
-            logger.setLevel(level)
+        if replace:
+            self.delete_collection(collection_name)
 
     def add_documents(self, documents: Sequence[Document]) -> None:
         super().maybe_add_ids(documents)
@@ -229,64 +202,52 @@ class LanceDB(VectorStore):
         coll_name = self.config.collection_name
         if coll_name is None:
             raise ValueError("No collection name set, cannot ingest docs")
+        # self._maybe_set_doc_class_schema(documents[0])
+        table_exists = False
         if (
-            coll_name not in colls
-            or self.client.open_table(coll_name).head(1).shape[0] == 0
+            coll_name in colls
+            and self.client.open_table(coll_name).head(1).shape[0] > 0
         ):
-            # collection either doesn't exist or is empty, so replace it,
-            # possibly with a new schema
-            extra_metadata_fields = extra_metadata(
-                documents[0], self.config.document_class
-            )
-            if len(extra_metadata_fields) > 0:
-                logger.warning(
-                    f"""
-                    Added documents contain extra metadata fields:
-                    {extra_metadata_fields}
-                    Trying to recreate the collection {coll_name} with a new schema:
-                    Overriding LanceDBConfig.document_class with an auto-generated 
-                    Pydantic class that includes these extra fields.
-                    If this fails, or you see odd results, it is recommended that you 
-                    define a subclass of Document, with metadata of class derived from 
-                    DocMetaData, with extra fields defined via 
-                    `Field(..., description="...")` declarations,
-                    and set this document class as the value of the 
-                    LanceDBConfig.document_class attribute.
-                    """
-                )
-
-                doc_cls = extend_document_class(documents[0])
-                self.config.document_class = doc_cls
-
-            doc_cls = self.config.document_class
-            self._setup_schemas(doc_cls)
-            self.create_collection(coll_name, replace=True)
+            # collection exists and  is not empty:
+            # if replace_collection is True, we'll overwrite the existing collection,
+            # else we'll append to it.
+            if self.config.replace_collection:
+                self.client.drop_table(coll_name)
+            else:
+                table_exists = True
 
         ids = [str(d.id()) for d in documents]
         # don't insert all at once, batch in chunks of b,
         # else we get an API error
         b = self.config.batch_size
 
-        def make_batches() -> Generator[List[BaseModel], None, None]:
+        def make_batches() -> Generator[List[Dict[str, Any]], None, None]:
             for i in range(0, len(ids), b):
                 batch = [
-                    self.unflattened_schema(
-                        id=ids[i],
-                        vector=embedding_vecs[i],
+                    dict(
+                        id=ids[i + j],
+                        vector=embedding_vecs[i + j],
                         **doc.dict(),
                     )
-                    for i, doc in enumerate(documents[i : i + b])
+                    for j, doc in enumerate(documents[i : i + b])
                 ]
-                if self.config.flatten:
-                    batch = [
-                        flatten_pydantic_instance(instance)  # type: ignore
-                        for instance in batch
-                    ]
                 yield batch
 
-        tbl = self.client.open_table(self.config.collection_name)
         try:
-            tbl.add(make_batches())
+            if table_exists:
+                tbl = self.client.open_table(coll_name)
+                tbl.add(make_batches())
+            else:
+                batch_gen = make_batches()
+                batch = next(batch_gen)
+                # use first batch to create table...
+                tbl = self.client.create_table(
+                    coll_name,
+                    data=batch,
+                    mode="create",
+                )
+                # ... and add the rest
+                tbl.add(batch_gen)
         except Exception as e:
             logger.error(
                 f"""
@@ -352,16 +313,17 @@ class LanceDB(VectorStore):
                 exclude=["vector"],
             )
             self.config.document_class = doc_cls  # type: ignore
-            self._setup_schemas(doc_cls)  # type: ignore
         else:
             # collection exists and is not empty, so append to it
             tbl = self.client.open_table(self.config.collection_name)
             tbl.add(df)
 
     def delete_collection(self, collection_name: str) -> None:
-        self.client.drop_table(collection_name)
+        self.client.drop_table(collection_name, ignore_missing=True)
 
-    def _lance_result_to_docs(self, result: LanceVectorQueryBuilder) -> List[Document]:
+    def _lance_result_to_docs(
+        self, result: "LanceVectorQueryBuilder"
+    ) -> List[Document]:
         if self.is_from_dataframe:
             df = result.to_pandas()
             return dataframe_to_documents(
@@ -375,41 +337,27 @@ class LanceDB(VectorStore):
             return self._records_to_docs(records)
 
     def _records_to_docs(self, records: List[Dict[str, Any]]) -> List[Document]:
-        if self.config.flatten:
-            docs = [
-                self.unflattened_schema(**nested_dict_from_flat(rec)) for rec in records
-            ]
-        else:
-            try:
-                docs = [self.schema(**rec) for rec in records]
-            except ValidationError as e:
-                raise ValueError(
-                    f"""
-                Error validating LanceDB result: {e}
-                HINT: This could happen when you're re-using an 
-                existing LanceDB store with a different schema.
-                Try deleting your local lancedb storage at `{self.config.storage_path}`
-                re-ingesting your documents and/or replacing the collections.
-                """
-                )
-
-        doc_cls = self.config.document_class
-        doc_cls_field_names = doc_cls.__fields__.keys()
-        return [
-            doc_cls(
-                **{
-                    field_name: getattr(doc, field_name)
-                    for field_name in doc_cls_field_names
-                }
+        try:
+            docs = [self.config.document_class(**rec) for rec in records]
+        except ValidationError as e:
+            raise ValueError(
+                f"""
+            Error validating LanceDB result: {e}
+            HINT: This could happen when you're re-using an
+            existing LanceDB store with a different schema.
+            Try deleting your local lancedb storage at `{self.config.storage_path}`
+            re-ingesting your documents and/or replacing the collections.
+            """
             )
-            for doc in docs
-        ]
+        return docs
 
     def get_all_documents(self, where: str = "") -> List[Document]:
         if self.config.collection_name is None:
             raise ValueError("No collection name set, cannot retrieve docs")
+        if self.config.collection_name not in self.list_collections(empty=True):
+            return []
         tbl = self.client.open_table(self.config.collection_name)
-        pre_result = tbl.search(None).where(where or None)
+        pre_result = tbl.search(None).where(where or None).limit(None)
         return self._lance_result_to_docs(pre_result)
 
     def get_documents_by_ids(self, ids: List[str]) -> List[Document]:
@@ -417,10 +365,11 @@ class LanceDB(VectorStore):
             raise ValueError("No collection name set, cannot retrieve docs")
         _ids = [str(id) for id in ids]
         tbl = self.client.open_table(self.config.collection_name)
-        docs = [
-            self._lance_result_to_docs(tbl.search().where(f"id == '{_id}'"))[0]
-            for _id in _ids
-        ]
+        docs = []
+        for _id in _ids:
+            results = self._lance_result_to_docs(tbl.search().where(f"id == '{_id}'"))
+            if len(results) > 0:
+                docs.append(results[0])
         return docs
 
     def similar_texts_with_scores(
@@ -432,7 +381,10 @@ class LanceDB(VectorStore):
         embedding = self.embedding_fn([text])[0]
         tbl = self.client.open_table(self.config.collection_name)
         result = (
-            tbl.search(embedding).metric(self.config.distance).where(where).limit(k)
+            tbl.search(embedding)
+            .metric(self.config.distance)
+            .where(where, prefilter=True)
+            .limit(k)
         )
         docs = self._lance_result_to_docs(result)
         # note _distance is 1 - cosine
